;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Wall Scaling bullet bill shooter.
;;
;; This bullet bill shooter will travel up and down between the
;; ceiling and the floor and will fire bullet bills at a set interval.
;; Based on the wall scaling cannon by Buster Beetle, and inspired
;; by the wall scrolling cannons in Zelda:ALTTP (Desert Palace).
;;
;; Uses first extra bit: YES
;; If extra bit is set, the shooter will fire towards Mario and use a different palette 
;; configurable below. If the bit is NOT set, then it will fire left or right
;; always, depending on X position.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;
;Symbolic names for RAM addresses
;;;;;;;;;;;;;;;;;;;;;;;;;
	RAM_ThrowTimer = $1504
	EXTRA_BITS = $7FAB10
;;;;;;;;;;
;CONFIGURABLES
;;;;;;;;;;
	TILE	= $EA		;Tile to use (Set GFX page with PAL1 and PAL2)
	PAL1	= $03		;Palette and GFX page for non-facing cannon (Even, page 1, odd, page 2)
	PAL2	= $01		;Palette and GFX page for cannon facing Mario (Even, page 1, odd, page 2)
	TIME_TILL_THROW     dcb $4a,$4a	  ;Time to Fire
	Y_SPEED		dcb $F0,$10	  ;Speed (Up,Down)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; INIT and MAIN JSL targets
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    dcb "INIT"
	LDA $7FAB10,x
	BIT #$04
	BNE FACEMARIO
 LDA $E4,x ; position
                    AND #$10
                    BNE RIGHT
LDA #$01
STA $157C,x
BRA CONTINUE_INIT
RIGHT
LDA #$00
STA $157C,x
BRA CONTINUE_INIT
FACEMARIO
                    JSR SUB_HORZ_POS
                    TYA
                    STA $157C,x
CONTINUE_INIT
                    TXA
					LDA #$35
                    STA RAM_ThrowTimer,x                 

                    RTL

                    dcb "MAIN"
					PHB                     ; \
                    PHK                     ;  | main sprite function, just calls local subroutine
                    PLB                     ;  |
	    JSR DecrementTimers
                    JSR SPRITE_ROUTINE      ;  |
                    PLB                     ;  |
                    RTL                     ; /
	
DecrementTimers:
	LDA $14C8,x
	CMP #$08
	BNE DoneDec
	LDA $9D
	BNE DoneDec
	LDA RAM_ThrowTimer,x
	BEQ DoneDec
	DEC RAM_ThrowTimer,x
DoneDec:
	RTS   

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SPRITE_ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SPRITE_ROUTINE	  	JSR SUB_GFX
                    LDA $14C8,x             ; return if sprite status != 8
                    CMP #$08            	 
                    BNE RETURN           
                    LDA $9D			        ; return if sprites locked
                    BNE RETURN    			
	
	LDA $7FAB10,x	;sub 17 from ypos if upside down
	BIT #$04
	BEQ NOFACE
					JSR SUB_HORZ_POS         ; \ always face mario
                    TYA                     ;  | 
                    STA $157C,x             ; /
NOFACE
LDA $1588,x		; \ if sprite is in contact with an object...
			AND #$0C		;  |
			BEQ NO_CONTACT_Y		;  |
			LDA $151C,x		;  | flip the direction status
			EOR #$01		;  |
			STA $151C,x		; /
NO_CONTACT_Y
			LDY $151C,x		; \ set x speed based on direction
			LDA Y_SPEED,y		;  |
			STA $AA,x		; /                    
                                        
			JSL $01802a		; update position based on speed values

	
NO_THROW
		
                    LDA RAM_ThrowTimer,x             ; \ Shoot if it's time					
                    BNE NO_RESET
	    LDY $C2,x
                    LDA TIME_TILL_THROW,y
                    STA RAM_ThrowTimer,x
					
NO_RESET            CMP #$01
                    BNE RETURN
                    LDA $C2,x
                    EOR #$01
                    STA $C2,x
	    
	    LDA $15A0,x             ; \ no Bullet if off screen
                    ORA $186C,x             ;  |
                    ORA $15D0,x	
	    BNE RETURN
					JSR SUB_BULLET
					LDA #$09
					STA $1DFC
					
RETURN              RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GRAPHICS ROUTINE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_GFX             JSR GET_DRAW_INFO       ; after: Y = index to sprite OAM ($300)
                                            ;      $00 = sprite x position relative to screen boarder 
                                            ;      $01 = sprite y position relative to screen boarder  
                    
                    LDA $157C,x             ; $02 = sprite direction
                    STA $02

                    LDA $00                 ; set x position of the tile
CLC
ADC #$00
                    STA $0300,y
   LDA $01                 ; \ tile y position = sprite y location ($01) + tile displacement
                    CLC                     ;  |
                    ADC #$08         ;  |
                    STA $0301,y             ; /

                  ;  LDA $01                 ; set y position of the tile
                    ;STA $0301,y

                    LDA #tile
                    STA $0302,y
	LDA $7FAB10,x	;sub 17 from ypos if upside down
	BIT #$04
	BNE NORMAL_PAL
LDA #PAL1
STA $15F6,x
BRA PALSET
NORMAL_PAL
LDA #PAL2
STA $15F6,x
PALSET
                    LDA $15F6,x             ; get sprite palette info
                    PHX
                    LDX $02                 ; flip the tile if the sprite direction is 0
                    BNE NO_FLIP_1
                    ORA #$40
NO_FLIP_1           PLX
                    ORA $64                 ; add in the priority bits from the level settings
                    STA $0303,y             ; set properties

                    INY                     ; get the index to the next slot of the OAM
                    INY                     ; (this is needed if you wish to draw another tile)
                    INY
                    INY
                    
                    LDY #$02                ; #$02 means the tiles are 16x16
                    LDA #$00                ; This means we drew one tile
                    JSL $01B7B3
                    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROUTINES FROM THE LIBRARY ARE PASTED BELOW
; You should never have to modify this code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1              dcb $0C,$1C
SPR_T2              dcb $01,$02

GET_DRAW_INFO       STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X         LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.W #$0040            ;  |
                    CMP.W #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP      LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y         DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID             PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Bullet Bill Routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPRITE_GEN = $1C

GENPOS	dcw $0004,$FFFA
GENPOSY	dcb $08,$08

SUB_BULLET
LDA $15A0,x             ; \ no Bullet if off screen
                    ORA $186C,x             ;  |
                    ORA $15D0,x	
BNE RETURNGC
	JSL $02A9E4		;grab free sprite slot
	BMI ReturnGC		;return if none left...

	LDA #$08
	STA $14C8,y		;normal status, run init routine?

	LDA #SPRITE_GEN		;load sprite to generate...
	STA $9E,y		;and restore  

	LDA $E4,x		;store 16bits xposition at $00
	STA $00
	LDA $14E0,x
	STA $01

	PHY
	JSR SUB_HORZ_POS	;genpos depending on size
	TYA
	ASL A			;16bits
	TAY
	REP #$20
	LDA GENPOS,y
	CLC
	ADC $00
	SEP #$20
	PLY

	STA $E4,y		;store xpositions
	XBA
	STA $14E0,y

	PHY
	JSR ExtrabitY
	LDA GENPOSY,y
	PLY
	CLC
	ADC $D8,x
	STA $D8,y		;Ypos
	LDA $14D4,x
	ADC #$00
	STA $14D4,y		;same screen

	PHX
	TYX			;new sprite slot into X
	JSL $07F7D2		;create
	PLX

	LDA $7FAB10,x	
	BIT #$04
	BEQ NOFACE2
	PHY
	JSR SUB_HORZ_POS	;direction based on mario side
	TYA
	PLY
	STA $C2,y
BRA FACE
NOFACE2
LDA $157C,x
STA $C2,y
FACE

	LDA $E4,y		;xpos low
	STA $00
	LDA $D8,y		;ypos low
	STA $01
	JSR SUB_SMOKE		;make some smoke

ReturnGC
	RTS  

;;;;;;;
;Smoke
;;;;;;;
SUB_SMOKE           LDY #$03                ; \ find a free slot to display effect
FINDFREE            LDA $17C0,y             ;  |
                    BEQ FOUNDONE            ;  |
                    DEY                     ;  |
                    BPL FINDFREE            ;  |
                    RTS                     ; / return if no slots open

FOUNDONE            LDA #$01                ; \ set effect graphic to smoke graphic
                    STA $17C0,y             ; /
                    LDA $01               ; \ smoke y position = generator y position
                    STA $17C4,y             ; /
                    LDA #$18                ; \ set time to show smoke
                    STA $17CC,y             ; /
                    LDA $00               ; \ load generator x position and store it for later
                    STA $17C8,y             ; /
                    LDX $15E9
             
                    RTS

ExtrabitY
	LDA $7FAB10,x	;put extra bit into Y
	AND #$04
	LSR A
	LSR A
	TAY
	RTS
		